# -*- coding: utf-8 -*-
"""
    revision.file_transfer
    ~~~~~~~~~~~~~~~~~~~~~~

    :copyright: (c) 2017 by COLORFUL BOARD Inc.
    :license: MIT, see LICENSE for more details.
"""

from __future__ import absolute_import

import os

from click import open_file, progressbar
from requests import get, post, put

__all__ = (
    "FileTransfer",
)


class FileTransfer(object):

    #: Transfer chunk size
    chunk_size = 1024

    def download(self,
                 url,
                 dest_path=None):
        """
        :param url:
        :type url: str
        :param dest_path:
        :type dest_path: str
        """
        if os.path.exists(dest_path):
            os.remove(dest_path)

        resp = get(url, stream=True)
        size = int(resp.headers.get("content-length"))
        label = "Downloading {filename} ({size:.2f}MB)".format(
            filename=os.path.basename(dest_path),
            size=size / float(self.chunk_size) / self.chunk_size
        )

        with open_file(dest_path, 'wb') as file:
            content_iter = resp.iter_content(chunk_size=self.chunk_size)
            with progressbar(content_iter,
                             length=size / self.chunk_size,
                             label=label) as bar:
                for chunk in bar:
                    if chunk:
                        file.write(chunk)
                        file.flush()

    def upload(self,
               url,
               method="POST",
               file_path=None):
        """
        :param url:
        :type url: str
        :param method:
        :type method: str
        :param file_path:
        :type file_path: str
        """
        if not os.path.exists(file_path):
            raise RuntimeError("")

        with open_file(file_path, 'rb') as file:
            size = os.path.getsize(file_path)
            label = "Uploading {filename} ({size:.2f}MB)".format(
                filename=os.path.basename(file_path),
                size=size / float(self.chunk_size) / self.chunk_size
            )

            if method == "PUT":
                resp = put(url, data=file)
            elif method == "POST":
                resp = post(url, data=file)

            content_iter = resp.iter_content(chunk_size=self.chunk_size)

            with progressbar(content_iter,
                             length=size / self.chunk_size,
                             label=label) as bar:
                for _ in bar:
                    pass



# -*- coding: utf-8 -*-
"""
    revision.client
    ~~~~~~~~~~~~~~~

    :copyright: (c) 2017 by COLORFUL BOARD Inc.
    :license: MIT, see LICENSE for more details.
"""

from __future__ import absolute_import

from functools import wraps
import os

from revision.archiver import Archiver
from revision.constants import (
    MESSAGE_LINE_SEPARATOR,
    REVISION_HOME,
    TMP_DIR
)
from revision.data import Revision
from revision.exceptions import InvalidArgType
from revision.file_transfer import FileTransfer
from revision.history import History
from revision.state import State

__all__ = (
    "download_required",
    "Client"
)


def download_required(func):
    @wraps(func)
    def decorator(self, *args, **kwargs):
        if not self.is_download:
            self.download(self)
        return func(self, *args, **kwargs)
    return decorator


class Client(object):

    #: Client config object.
    config = None

    #: Zip archiver object.
    #: See :class:`revision.archiver.Archiver` for more information.
    archiver = None

    #: File transfer object.
    #: See :class:`revision.file_transfer.FileTransfer` for more information.
    transfer = None

    #: The client revision history.
    #: See :class:`revision.history.History` for more information.
    history = None

    state = None

    def __init__(self, config=None):
        """
        :param config:
        :type config: dict
        """
        self.archiver = Archiver()
        self.transfer = FileTransfer()
        self.history = History()
        self.state = State.create()

        if config:
            self.add_config(config)

    def add_config(self, config):
        """
        :param config:
        :type config: dict
        """
        self.pre_configure()

        self.config = config

        self.archiver.target_path = self.dest_path
        self.archiver.zip_path = self.tmp_file_path

        if self.has_revision_file():
            self.history.load(self.revfile_path)

        self.state.state_path = os.path.join(
            REVISION_HOME,
            "clients",
            self.key
        )
        self.state.prepare()

        self.post_configure()

    def pre_configure(self):
        pass

    def post_configure(self):
        pass

    @property
    def name(self):
        """
        :return:
        :rtype: str
        """
        raise NotImplementedError()

    @property
    def key(self):
        """
        :return:
        :rtype: str
        """
        return self.config.key

    @property
    def client_key(self):
        """
        :return:
        :rtype: str
        """
        raise NotImplementedError()

    @property
    def is_download(self):
        """
        :return:
        :rtype: boolean
        """
        return os.path.isdir(self.dest_path)

    @property
    def filename(self):
        """
        :return:
        :rtype: str
        """
        return "{}.zip".format(
            self.key
        )

    def has_revision_file(self):
        """
        :return:
        :rtype: boolean
        """
        return os.path.exists(self.revfile_path)

    def has_info_file(self):
        """
        :return:
        :rtype: boolean
        """
        return os.path.exists(self.infofile_path)

    def has_commit(self):
        """
        :return:
        :rtype: boolean
        """
        current_revision = self.history.current_revision
        revision_id = self.state.revision_id

        return current_revision.revision_id == revision_id

    @property
    def revfile_path(self):
        """
        :return:
        :rtype: str
        """
        return os.path.normpath(os.path.join(
            self.dest_path,
            self.config.revision_file
        ))

    @property
    def infofile_path(self):
        """
        :return:
        :rtype: str
        """
        return os.path.normpath(os.path.join(
            self.dest_path,
            self.config.info_file
        ))

    @property
    def dest_path(self):
        """
        :return:
        :rtype: str
        """
        return os.path.normpath(os.path.join(
            os.getcwd(),
            self.config.dir_path
        ))

    @property
    def tmp_file_path(self):
        """
        :return:
        :rtype: str
        """
        return os.path.normpath(os.path.join(
            TMP_DIR,
            self.filename
        ))

    def save(self, revision):
        """
        :param revision:
        """
        if not isinstance(revision, Revision):
            raise InvalidArgType()

        self.state.update(revision)

    def write(self):
        revision = Revision(
            revision_id=self.state.revision['revision_id'],
            release_date=self.state.revision['release_date'],
            description=self.state.revision['description'],
            message=self.state.revision['message']
        )

        self.history.prepend(revision)

        with open(self.revfile_path, 'w') as f:
            f.write("# CHANGELOG" + MESSAGE_LINE_SEPARATOR)
            for revision in self.history.revisions:
                f.write(revision.to_markdown())

    def download(self):
        raise NotImplementedError()

    def upload(self):
        raise NotImplementedError()

    def __repr__(self):
        return "<class {}.{}> " \
               "name: {}, " \
               "key: {}, " \
               "config: {}".format(
                   self.__module__,
                   self.__class__.__name__,
                   self.name,
                   self.key,
                   self.config)



# -*- coding: utf-8 -*-
"""
    revision.config
    ~~~~~~~~~~~~~~~

    :copyright: (c) 2017 by COLORFUL BOARD Inc.
    :license: MIT, see LICENSE for more details.
"""

from __future__ import absolute_import

import json
import os

from revision.exceptions import (
    ConfigNotFound,
    MissingConfigValue
)
from revision.mixins import DotDictMixin

__all__ = (
    "DEFAULT_CONFIG_FILENAME",
    "DEFAULT_CONFIG_TMPL",
    "Config",
    "read_config",
)


DEFAULT_CONFIG_FILENAME = ".revision.json"

DEFAULT_CONFIG_TMPL = {
    "clients": []
}

DEFAULT_REVISION_FILENAME = "CHANGELOG.md"


config = None


class Config(DotDictMixin):

    def prepare(self):
        for client in self.clients:
            if 'revision_file' not in client:
                client.revision_file = DEFAULT_REVISION_FILENAME

            if 'key' not in client:
                raise MissingConfigValue()

            if 'module' not in client:
                raise MissingConfigValue()

    def __repr__(self):
        obj = '{' + ', '.join('%r: %r' % i for i in self.iteritems()) + '}'
        return "<class 'revision.config.Config'> {}".format(obj)


def read_config(config_path_or_dict=None):
    """
    Read config from given path string or dict object.

    :param config_path_or_dict:
    :type config_path_or_dict: str or dict
    :return: The config object or None.
    :rtype: :class:`revision.config.Config`
    """
    global config

    if config is not None:
        return config

    if type(config_path_or_dict) == dict:
        config = Config(config_path_or_dict)

    if type(config_path_or_dict) == str:
        config_path = config_path_or_dict
    else:
        config_path = os.path.join(
            os.getcwd(),
            DEFAULT_CONFIG_FILENAME
        )

    if os.path.exists(config_path):
        with open(config_path, 'r') as f:
            data = json.load(f)
            config = Config(data)

    if config is None:
        raise ConfigNotFound()
    else:
        config.prepare()

        return config



# -*- coding: utf-8 -*-
"""
    revision.data
    ~~~~~~~~~~~~~

    :copyright: (c) 2017 by COLORFUL BOARD Inc.
    :license: MIT, see LICENSE for more details.
"""

from __future__ import absolute_import

import datetime
import json

from revision.constants import (
    DATETIME_FORMAT,
    MESSAGE_LINE_SEPARATOR
)
from revision.util import make_hash_id

__all__ = (
    "Revision",
)

re_datetime_str = "^(\d{4})/(\d{2})/(\d{2}) (\d{2}):(\d{2}):(\d{2})"


class Revision(object):

    #: Revision ID
    revision_id = None

    #: Release Date
    release_date = None

    #: Commit description
    description = ""

    #: Commit message
    message = ""

    def __init__(self,
                 revision_id=None,
                 release_date=None,
                 description="",
                 message=""):
        """
        :param revision_id:
        :type revision_id: str
        :param release_date:
        :type release_date: datetime
        :param description:
        :type description: str
        :param message:
        :type message: str
        """
        self.revision_id = revision_id
        self.description = description
        self.message = message

        if release_date is not None:
            self.release_date = datetime.datetime.strptime(
                release_date,
                DATETIME_FORMAT
            )

    @classmethod
    def create(cls, description="", message=""):
        """
        :param description:
        :type description: str
        :param message:
        :type message: str
        """
        instance = cls()
        instance.revision_id = make_hash_id()
        instance.release_date = datetime.datetime.now()

        if len(description) > 0:
            instance.description = description

        if len(message) > 0:
            instance.message = message

        return instance

    def parse(self, rev_string):
        """
        :param rev_string:
        :type rev_string: str
        """
        elements = rev_string.split(MESSAGE_LINE_SEPARATOR)

        heading = elements[0]

        heading_elements = heading.split(" ")

        self.revision_id = heading_elements[2]
        datetime_str = "{} {}".format(
            heading_elements[0],
            heading_elements[1]
        )
        self.release_date = datetime.datetime.strptime(
            datetime_str,
            DATETIME_FORMAT
        )

        self.description = elements[1]
        self.message = elements[2]

    def to_json(self):
        return json.dumps({
            'revision_id': self.revision_id,
            'release_date': self.release_date.strftime(DATETIME_FORMAT),
            'description': self.description,
            'message': self.message
        }, indent=2)

    def to_markdown(self):
        """
        :return:
        :rtype: str
        """
        return "## {} {}\n\n{}\n\n{}\n\n".format(
            self.release_date.strftime(DATETIME_FORMAT),
            self.revision_id,
            self.description,
            self.message
        )

    def has_description(self):
        """
        :return:
        :rtype: boolean
        """
        return len(self.description) > 0

    def has_message(self):
        """
        :return:
        :rtype: boolean
        """
        return len(self.message) > 0

    def __repr__(self):
        """
        :return:
        :rtype: str
        """
        if len(self.description) > 10:
            desc = self.description[:10] + '...'
        else:
            desc = self.description

        if len(self.message) > 10:
            msg = self.message[:10] + '...'
        else:
            msg = self.message

        return "<class 'revision.data.revision.Revision'> " \
               "id: {}, " \
               "date: {}, " \
               "desc: {}, " \
               "message: {}".format(
                   self.revision_id,
                   self.release_date,
                   desc,
                   msg)



# -*- coding: utf-8 -*-
"""
    revision.state
    ~~~~~~~~~~~~~~

    :copyright: (c) 2017 by COLORFUL BOARD Inc.
    :license: MIT, see LICENSE for more details.
"""

import json
import os

__all__ = (
    "State",
)


class State(object):

    state_path = None

    rev_id = None

    revision = None

    @classmethod
    def create(cls):
        klass = cls()

        return klass

    def prepare(self):
        if not os.path.isdir(self.state_path):
            os.makedirs(self.state_path)

        if not os.path.exists(self.head_path):
            self._touch(self.head_path)

        if os.path.exists(self.revision_path):
            with open(self.revision_path, 'r') as f:
                text = f.read()
                self.revision = json.loads(text)
        else:
            self._touch(self.revision_path)

    def clear(self):
        if os.path.exists(self.revision_path):
            os.remove(self.revision_path)

        if os.path.exists(self.head_path):
            os.remove(self.head_path)

        if os.path.isdir(self.state_path):
            os.rmdir(self.state_path)

    @property
    def revision_id(self):
        if self.rev_id is None:
            with open(self.head_path, 'r') as f:
                self.rev_id = f.read()

        return self.rev_id

    def has_message(self):
        pass

    @property
    def head_path(self):
        return os.path.join(self.state_path, "HEAD")

    @property
    def revision_path(self):
        return os.path.join(self.state_path, "REVISION")

    def update(self, revision):
        with open(self.head_path, 'w') as f:
            f.write(revision.revision_id)

        with open(self.revision_path, 'w') as f:
            f.write(revision.to_json())

    def _touch(self, file_path):
        open(file_path, "a").close()



# -*- coding: utf-8 -*-
"""
    revision.exceptions
    ~~~~~~~~~~~~~~~~~~~

    :copyright: (c) 2017 by COLORFUL BOARD Inc.
    :license: MIT, see LICENSE for more details.
"""

from __future__ import absolute_import

__all__ = (
    "ClientNotExist",
    "ConfigNotFound",
    "InvalidArgType",
    "MissingConfigValue"
)


class ClientNotExist(Exception):

    description = (
        "client does not exist."
    )


class ConfigNotFound(Exception):

    description = (
        "config file does not found."
    )


class InvalidArgType(Exception):

    description = (
        "invalid type argument of "
    )


class MissingConfigValue(Exception):

    description = (
        "missing config value "
    )



# -*- coding: utf-8 -*-
"""
    revision.decorators
    ~~~~~~~~~~~~~~~~~~~

    :copyright: (c) 2017 by COLORFUL BOARD Inc.
    :license: MIT, see LICENSE for more details.
"""

from __future__ import absolute_import

from functools import wraps

from click.globals import get_current_context

from revision.orchestrator import Orchestrator

__all__ = (
    "pass_orchestrator",
)


def pass_orchestrator(func):
    @wraps(func)
    def decorator(*args, **kwargs):
        orchestrator = Orchestrator()

        ctx = get_current_context()
        client_key = ctx.obj.get('client_key', None)
        if client_key and orchestrator.clients.has_client(client_key):
            orchestrator.use(client_key)

        kwargs["orchestrator"] = orchestrator

        return func(*args, **kwargs)
    return decorator



# -*- coding: utf-8 -*-
"""
    revision.client_manager
    ~~~~~~~~~~~~~~~~~~~~~~~

    :copyright: (c) 2017 by COLORFUL BOARD Inc.
    :license: MIT, see LICENSE for more details.
"""

from revision.client import Client

__all__ = (
    "ClientManager",
)


class ClientManager(dict):

    def __init__(self, clients=None):
        """
        :param clients:
        :type clients: list
        """
        if isinstance(clients, list):
            for config in clients:
                self.add_client(self.instantiate_client(config))

    def instantiate_client(self, config):
        """
        :param config:
        :type config: dict
        :return: The instantiated class.
        :rtype: :class:`revision.client.Client`
        """
        modules = config.module.split('.')
        class_name = modules.pop()
        module_path = '.'.join(modules)

        client_instance = getattr(
            __import__(module_path, {}, {}, ['']),
            class_name
        )()

        client_instance.add_config(config)

        return client_instance

    def has_client(self, client_key):
        """
        :param client_key: The client key to check.
        :type client_key: str
        :return: Returns True if client_key is found.
        :rtype: boolean
        """
        return client_key in self

    def get_client(self, client_key):
        """
        :param client_key: The client key to get.
        :type client_key: str
        :return:
        :rtype: :class:`revision.client.Client`
        """
        return self.get(client_key, None)

    def add_client(self, client):
        """
        :param client:
        :type client: :class:`revision.client.Client`
        :return:
        :rtype: :class:`revision.client_manager.ClientManager`
        """
        if not isinstance(client, Client):
            return

        if self.has_client(client.key):
            return

        self[client.key] = client

        return self



# -*- coding: utf-8 -*-
"""
    revision.cli
    ~~~~~~~~~~~~

    :copyright: (c) 2017 by COLORFUL BOARD Inc.
    :license: MIT, see LICENSE for more details.
"""

from __future__ import absolute_import

import json
import os
import sys

from click import (
    echo,
    edit,
    group,
    style
)

from revision.config import (
    DEFAULT_CONFIG_FILENAME,
    DEFAULT_CONFIG_TMPL
)
from revision.constants import (
    MESSAGE_LINE_SEPARATOR,
    MESSAGE_NEW_LINE,
    MESSAGE_TEMPLATE
)
from revision.data import Revision
from revision.decorators import pass_orchestrator

__all__ = (
    "main",
)


@group()
def cli():
    pass


@cli.command()
def init():
    if os.path.exists(DEFAULT_CONFIG_FILENAME):
        echo(style(
            "{} file always exists.".format(DEFAULT_CONFIG_FILENAME),
            fg="green"
        ), err=True)
    else:
        with open(DEFAULT_CONFIG_FILENAME, "w") as f:
            json.dump(DEFAULT_CONFIG_TMPL, f, indent=2)


@cli.command()
@pass_orchestrator
def commit(orchestrator):
    if orchestrator.current_client:
        message = edit(MESSAGE_TEMPLATE)

        if message is None:
            return

        lines = message.split(MESSAGE_LINE_SEPARATOR)
        description = lines[1].strip(MESSAGE_NEW_LINE)
        message = lines[2].strip(MESSAGE_NEW_LINE)

        revision = Revision.create(
            description=description,
            message=message
        )

        orchestrator.commit(revision)
    else:
        echo(style(
            "please specify `client_key` for commit command.",
            fg="green"
        ), err=True)


@cli.command()
@pass_orchestrator
def push(orchestrator):
    if orchestrator.current_client:
        if orchestrator.has_commit():
            orchestrator.push()
        else:
            echo(style(
                "",
                fg="green"
            ), err=True)
    else:
        echo(style(
            "please specify `client_key` for push command.",
            fg="green"
        ), err=True)


@cli.command()
@pass_orchestrator
def pull(orchestrator):
    if orchestrator.current_client:
        orchestrator.current_client.download()
    else:
        for key, _ in orchestrator.clients:
            if not orchestrator.clients.has_client(key):
                continue

            client = orchestrator.clients.get_client(key)
            client.download()


def main():
    client_key = None

    if len(sys.argv) >= 2 and not (sys.argv[1] in cli.commands.keys()):
        client_key = sys.argv[1]

        sys.argv.pop(1)

    cli(obj={
        "client_key": client_key
    })


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""
    revision.mixins
    ~~~~~~~~~~~~~~~

    :copyright: (c) 2017 by COLORFUL BOARD Inc.
    :license: MIT, see LICENSE for more details.
"""

__all__ = (
    "DotDictMixin",
)


class DotDictMixin(dict):
    """
    This mixin provides dot notation access.
    """

    def __init__(self, *args, **kwargs):
        super(DotDictMixin, self).__init__(*args, **kwargs)

        for arg in args:
            if isinstance(arg, dict):
                for k, v in arg.iteritems():
                    if isinstance(v, list):
                        new_list = []
                        for a in v:
                            if isinstance(a, dict):
                                a = DotDictMixin(a)
                            new_list.append(a)
                        v = new_list
                    elif isinstance(v, dict):
                        v = DotDictMixin(v)

                    self[k] = v

        if kwargs:
            for k, v in kwargs.iteritems():
                self[k] = v

    def __getattr__(self, attr):
        return self.get(attr)

    def __setattr__(self, attr, val):
        self[attr] = val



# -*- coding: utf-8 -*-
"""
    revision
    ~~~~~~~~

    :copyright: (c) 2017 by COLORFUL BOARD Inc.
    :license: MIT, see LICENSE for more details.
"""

from __future__ import absolute_import

__version__ = "0.0.3"
__license__ = "MIT"



# -*- coding: utf-8 -*-
"""
    revision.history
    ~~~~~~~~~~~~~~~~

    :copyright: (c) 2017 by COLORFUL BOARD Inc.
    :license: MIT, see LICENSE for more details.
"""

from __future__ import absolute_import

import os

from revision.data import Revision

__all__ = (
    "History",
)


class History(object):

    #: Revisions
    revisions = []

    current_index = None

    def __init__(self, revisions=[], current_index=None):
        self.revisions = revisions
        self.current_index = current_index

    @property
    def current_revision(self):
        """
        :return: The current :class:`revision.data.Revision`.
        :rtype: :class:`revision.data.Revision`
        """
        if self.current_index is None:
            return None

        return self.revisions[self.current_index]

    def load(self, revision_path):
        """
        Load resision file.

        :param revision_path:
        :type revision_path: str
        """
        if not os.path.exists(revision_path):
            raise RuntimeError("")

        with open(revision_path, mode='r') as f:
            text = f.read()
            rev_strings = text.split("## ")

            for rev_string in rev_strings:
                if len(rev_string) == 0 or rev_string[:2] == "# ":
                    continue

                try:
                    revision = Revision()
                    revision.parse(rev_string)
                except RuntimeError:
                    raise RuntimeError("")

                self.insert(revision, len(self.revisions))

    def clear(self):
        """
        Clear :class:`revision.history.History` states.
        """
        self.revisions = []
        self.current_index = 0

    def checkout(self, revision_id):
        """
        :param revision_id: :class:`revision.data.Revision` ID.
        :type revision_id: str
        """
        index = 0
        found = False
        for revision in self.revisions:
            if revision.revision_id == revision_id:
                self.current_index = index
                found = True

            index += 1

        if not found:
            raise RuntimeError("")

    def prepend(self, revision):
        """
        Prepend a :class:`revision.data.Revision` to the beginning of
        a revisions.

        :param revision:
        :type revision: :class:`revision.data.Revision`
        """
        return self.insert(revision, 0)

    def insert(self, revision, index):
        """
        Insert a :class:`revision.data.Revision` at a given index.

        :param revision:
        :type revision: :class:`revision.data.Revision`
        :param index:
        :type index: int
        """
        if not isinstance(revision, Revision):
            raise RuntimeError("")

        self.revisions.insert(index, revision)

        return self



# -*- coding: utf-8 -*-
"""
    revision.orchestrator
    ~~~~~~~~~~~~~~~~~~~~~

    :copyright: (c) 2017 by COLORFUL BOARD Inc.
    :license: MIT, see LICENSE for more details.
"""

from __future__ import absolute_import

import os

from revision.client_manager import ClientManager
from revision.config import read_config
from revision.data import Revision
from revision.exceptions import (
    ClientNotExist,
    ConfigNotFound,
    InvalidArgType
)

__all__ = (
    "Orchestrator",
)


class Orchestrator(object):

    #: The root path of the project.
    project_root_path = None

    #: The class that is used for client instances.
    #: See :class:`revision.client_manager.ClientManager` for more information.
    clients = None

    #: The project configuration.
    #: See :class:`revision.config.Config` for more information.
    config = None

    current_client = None

    def __init__(self, config_path_or_dict=None):
        """
        :param config_path_or_dict:
        """
        self.project_root_path = os.getcwd()

        try:
            self.config = read_config(config_path_or_dict)
        except ConfigNotFound as e:
            raise RuntimeError(e.message)

        self.clients = ClientManager(self.config.clients)

    def use(self, client_key):
        """
        :param client_key:
        :type client_key: str
        """
        if not self.clients.has_client(client_key):
            raise ClientNotExist()

        self.current_client = self.clients.get_client(client_key)

    def commit(self, revision):
        """
        :param revision:
        :type revision: :class:`revision.data.Revision`
        """
        if not isinstance(revision, Revision):
            raise InvalidArgType()

        if not self.current_client:
            return

        self.current_client.save(revision)

    def push(self):
        """
        """
        self.current_client.write()
        self.current_client.upload()

    def pull(self):
        pass

    def has_commit(self, client_key=None):
        """
        :param client_key:
        :type client_key: str
        :return:
        :rtype: boolean
        """
        return True
        # if client_key is None and self.current_client is None:
        #     raise ClientNotExist()

        # if client_key:
        #     if not self.clients.has_client(client_key):
        #         raise ClientNotExist()

        #     client = self.clients.get_client(client_key)

        #     return client.has_commit()

        # if self.current_client:
        #     client = self.current_client

        #     return client.has_commit()

        # return False

    def __repr__(self):
        result = "<class 'revision.orchestrator.Orchestrator'>"

        if self.current_client:
            result += " current client: %s" % self.current_client.key

        return result



# -*- coding: utf-8 -*-
"""
    revision.constants
    ~~~~~~~~~~~~~~~~~~

    :copyright: (c) 2017 by COLORFUL BOARD Inc.
    :license: MIT, see LICENSE for more details.
"""

import os

__all__ = (
    "ARCHIVE_IGNORE_FILES",
    "DATETIME_FORMAT",
    "MESSAGE_LINE_SEPARATOR",
    "MESSAGE_NEW_LINE",
    "MESSAGE_TEMPLATE",
    "REVISION_HOME",
    "TMP_DIR"
)


ARCHIVE_IGNORE_FILES = [
    ".gitkeep",
    ".DS_Store"
]

DATETIME_FORMAT = "%Y/%m/%d %H:%M:%S"

MESSAGE_LINE_SEPARATOR = "\n\n"

MESSAGE_NEW_LINE = "\n"

MESSAGE_TEMPLATE = """# Please enter the commit message for your changes.

[DESCRIPTION]

[MESSAGE]
"""

REVISION_HOME = os.path.expanduser('~/.revision')

TMP_DIR = "/tmp"



# -*- coding: utf-8 -*-
"""
    revision.test
    ~~~~~~~~~~~~~

    :copyright: (c) 2017 by COLORFUL BOARD Inc.
    :license: MIT, see LICENSE for more details.
"""

from revision.client import Client

__all__ = (
    "DummyClient",
)


class DummyClient(Client):

    @property
    def name(self):
        return "Dummy storage for testing"

    @property
    def client_key(self):
        return "test"

    def download(self):
        pass

    def upload(self):
        pass



# -*- coding: utf-8 -*-
"""
    revision.util
    ~~~~~~~~~~~~~

    :copyright: (c) 2017 by COLORFUL BOARD Inc.
    :license: MIT, see LICENSE for more details.
"""

from __future__ import absolute_import

import datetime
import hashlib

from revision.constants import DATETIME_FORMAT

__all__ = (
    "make_hash_id",
)


def make_hash_id():
    """
    Compute the `datetime.now` based SHA-1 hash of a string.

    :return: Returns the sha1 hash as a string.
    :rtype: str
    """
    today = datetime.datetime.now().strftime(DATETIME_FORMAT)
    return hashlib.sha1(today).hexdigest()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re
import ast
from setuptools import find_packages, setup

_version_re = re.compile(r'__version__\s+=\s+(.*)')

with open('revision/__init__.py', 'rb') as f:
    version = str(ast.literal_eval(_version_re.search(
        f.read().decode('utf-8')
    ).group(1)))

setup(
    name='revision',
    version=version,
    author='COLORFUL BOARD Inc.',
    url='https://github.com/COLORFULBOARD/revision',
    license='MIT',
    packages=find_packages(),
    install_requires=[
        'click==6.7',
        'requests==2.18.4'
    ],
    extras_require={
        'dev': [
            'flake8==3.4.1',
            'pytest==3.2.2'
        ]
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    entry_points={'console_scripts': [
        'revision=revision.cli:main'
    ]}
)



# -*- coding: utf-8 -*-
"""
    revision.archiver
    ~~~~~~~~~~~~~~~~~

    :copyright: (c) 2017 by COLORFUL BOARD Inc.
    :license: MIT, see LICENSE for more details.
"""

from __future__ import absolute_import

import os
import zipfile

from revision.constants import ARCHIVE_IGNORE_FILES

__all__ = (
    "Archiver",
)


class Archiver(object):

    target_path = None

    zip_path = None

    def __init__(self, target_path=None, zip_path=None):
        """
        :param target_path:
        :type target_path: str
        :param zip_path: The file path of the ZIP archive.
        :type zip_path: str
        """
        self.target_path = target_path
        self.zip_path = zip_path

    @property
    def has_path(self):
        """
        :return: Checks whether path exists.
        :rtype: boolean
        """
        return (self.target_path is not None) and (self.zip_path is not None)

    def archive(self, target_path=None, zip_path=None):
        """
        Adds a file to a ZIP archive from a given path.

        :param target_path: The directory path to add.
        :type target_path: str
        :param zip_path: The file path of the ZIP archive.
        :type zip_path: str
        """
        if target_path:
            self.target_path = target_path

        if zip_path:
            self.zip_path = zip_path

        if self.has_path is False or os.path.isdir(self.target_path) is False:
            raise RuntimeError("")

        zip = zipfile.ZipFile(
            self.zip_path,
            'w',
            zipfile.ZIP_DEFLATED
        )

        for root, _, files in os.walk(self.target_path):
            for file in files:
                if file in ARCHIVE_IGNORE_FILES:
                    continue

                current_dir = os.path.relpath(root, self.target_path)

                if current_dir == ".":
                    file_path = file
                else:
                    file_path = os.path.join(current_dir, file)

                print "Archive {}".format(file)

                zip.write(
                    os.path.join(root, file),
                    file_path
                )

        zip.close()

    def unarchive(self, target_path=None, zip_path=None):
        """
        Extract the given files to the specified destination.

        :param src_path: The destination path where to extract the files.
        :type src_path: str
        :param zip_path: The file path of the ZIP archive.
        :type zip_path: str
        """
        if target_path:
            self.target_path = target_path

        if zip_path:
            self.zip_path = zip_path

        if self.has_path is False:
            raise RuntimeError("")

        if os.path.isdir(self.target_path) is False:
            os.mkdir(self.target_path)

        with zipfile.ZipFile(self.zip_path, 'r') as zip:
            zip.extractall(self.target_path)
